/*------------------------------------------------------------------------
Copyright LIRIS, M2DisCo 2009, Licence QPL
Author: Guillaume Lavoue
------------------------------------------------------------------------*/

#ifndef DISTANCE_3D_H
#define DISTANCE_3D_H


#include <CGAL/circulator.h>
#include <CGAL/HalfedgeDS_decorator.h>
#include <CGAL/HalfedgeDS_face_base.h> 
#include<set>
#include<map>
#include<list>
#include<vector>

#include<set>
#include<map>
#include<list>
#include<stack>
#include <algorithm>



#include <time.h>
//#include"matrix.h"

#define myPI 3.1415926535



template <class _Poly>
class Distance3D
{
  typedef _Poly                                        Polyhedron;

  typedef typename Polyhedron::Halfedge_data_structure HDS;
  typedef typename Polyhedron::Vertex                  Vertex;
  typedef typename Polyhedron::Halfedge                Halfedge;
  typedef typename Polyhedron::Facet                   Facet;
  
  typedef typename Polyhedron::Vertex_handle           Vertex_handle;
  typedef typename Polyhedron::Halfedge_handle         Halfedge_handle;
  typedef typename Polyhedron::Facet_handle            Facet_handle;

  typedef typename Polyhedron::Vertex_iterator         Vertex_iterator;
  typedef typename Polyhedron::Halfedge_iterator       Halfedge_iterator;
  typedef typename Polyhedron::Edge_iterator           Edge_iterator;
  typedef typename Polyhedron::Facet_iterator          Facet_iterator;

  typedef typename Polyhedron::Halfedge_around_facet_circulator  
                                            Halfedge_around_facet_circulator;
  typedef typename Polyhedron::Halfedge_around_vertex_circulator 
                                            Halfedge_around_vertex_circulator;
  typedef typename Polyhedron::Point_3                 Point;
  typedef typename Polyhedron::FT                           FT;
  typedef typename Polyhedron::Vector                       Vector;

  
private:

	

	
	bool m_IsLoaded,is_calculated;
	FILE * times;
	
public:	

	Polyhedron *m_PolyOriginal;
	Polyhedron m_PolyDegrad;

	bool IsMapDistanceCalculated;
	Distance3D()
	{
		IsMapDistanceCalculated=false;

		m_IsLoaded=false;
		
}

	Distance3D(Polyhedron *_PolyOriginal)
	{
		PolyOriginal(_PolyOriginal);
		m_IsLoaded=false;
		
	}


	const bool& IsLoaded() const { return m_IsLoaded;	}
	bool& IsLoaded() { return m_IsLoaded;	}
	void IsLoaded(const bool& p)	{	m_IsLoaded	=	p;}

	const Polyhedron& PolyOriginal() const { return *m_PolyOriginal;	}
	Polyhedron& PolyOriginal() { return *m_PolyOriginal;	}
	void PolyOriginal(Polyhedron* p)	{	m_PolyOriginal	=	p; }

	const Polyhedron& PolyDegrad() const { return m_PolyDegrad;	}
	Polyhedron& PolyDegrad() { return m_PolyDegrad;	}
	void PolyDegrad(const Polyhedron& p)	{
		
		m_PolyDegrad	=	p;
		for(Facet_iterator pFacet	=	m_PolyDegrad.facets_begin();pFacet!= m_PolyDegrad.facets_end();pFacet++)
		{
			Halfedge_around_facet_circulator pHalfedge = pFacet->facet_begin();
			int size=CGAL::circulator_size(pHalfedge);
						
			if(size==4)
			{
				Halfedge_iterator V1=pHalfedge;
				Halfedge_iterator V2=pHalfedge->next()->next();
				m_PolyDegrad.split_facet(V1,V2);
			}
						
		}
	
	
	}

	double Processroughness_curve_Dual(double radius, double maxdim,bool IsGauss = true)
	{
		
		m_PolyOriginal->MinNrmMeanCurvature(100000000);
		m_PolyOriginal->MaxNrmMeanCurvature(0);

		m_PolyOriginal->MinNrmSalCurvature(100000000);
		m_PolyOriginal->MaxNrmSalCurvature(0);

		m_PolyDegrad.MinNrmMeanCurvature(100000000);
		m_PolyDegrad.MaxNrmMeanCurvature(0);

		m_PolyDegrad.MinNrmSalCurvature(100000000);
		m_PolyDegrad.MaxNrmSalCurvature(0);

		double somme_roughness=0;
		int NbVert=0;

		Vertex_iterator	pVertexDeg=m_PolyDegrad.vertices_begin();
		for(Vertex_iterator	pVertex	=	m_PolyOriginal->vertices_begin();
					pVertex	!= m_PolyOriginal->vertices_end();
					pVertex++)
		{
				std::vector<double> TabDistance;
				std::vector<Point> TabPoint;

				std::vector<double> TabDistanceDeg;
				std::vector<Point> TabPointDeg;

				double moyenne,moyenneDeg;

				double var1=Processroughness_per_vertex_curve(m_PolyOriginal,(&(*pVertex)),radius,TabDistance,TabPoint,moyenne,maxdim,IsGauss);

				double var2=Processroughness_per_vertex_curve(&m_PolyDegrad,(&(*pVertexDeg)),radius,TabDistanceDeg,TabPointDeg,moyenneDeg,maxdim,IsGauss);

				double cov=ProcessCovariance((&(*pVertexDeg)),moyenne,moyenneDeg,TabDistance,TabPoint,TabDistanceDeg,TabPointDeg,maxdim,IsGauss);

				//double cov=1;
				pVertexDeg->CourbureCoVariance=pVertex->CourbureCoVariance=cov;

			

				pVertexDeg++;
				

		}
		return 0;
	}

	double ProcessCovariance(Vertex* pVertex,double moyenne,double moyenneDeg,std::vector<double> TabDistance,std::vector<Point> TabPoint,std::vector<double> TabDistanceDeg,std::vector<Point> TabPointDeg, double dim,bool IsGauss)
	{
		double Cov1,Cov2;
		double somme1=0;
		double somme2=0;
		int Nb1,Nb2;
		double x1,x2;

		double buff,maxBuff;
		Vector VBuff;

		int IndMax;
		int taille1=TabDistance.size();
		int taille2=TabDistanceDeg.size();
		if(taille1==0 || taille2==0)
			int h=3;

		int ecart=abs(taille1-taille2);

		double sommewi=0;
		for(int i=0;i<TabDistance.size();i++)
		{
			int IndMin=i-2*ecart;
			if(IndMin<0)
				IndMin=0;

			int IndMax=i+2*ecart;
			if(IndMax>TabPointDeg.size()-1)
				IndMax=TabPointDeg.size()-1;


			x1=TabDistance[i];
			x2=TabDistanceDeg[IndMin];
			VBuff=TabPointDeg[IndMin]-TabPoint[i];
			maxBuff=sqrt(VBuff*VBuff);
			

			
			for(int j=IndMin+1;j<=IndMax;j++)
			{
				VBuff=TabPointDeg[j]-TabPoint[i];
				buff=sqrt(VBuff*VBuff);
				if(buff<maxBuff)
				{
					
					maxBuff=buff;
					x2=TabDistanceDeg[j];

				}

			}

			if(IsGauss==false)
			{
				somme1+=(x1-moyenne)*(x2-moyenneDeg);
				sommewi+=1;
			}
			else
			{
				Vector DistancePt=TabPoint[i]-pVertex->point();
				double distPt=sqrt(DistancePt*DistancePt);
				double wi=1/0.008/dim/sqrt(2*3.141592)*exp(-(distPt*distPt)/2/0.008/0.008/dim/dim);

				sommewi+=wi;
				somme1+=wi*(x1-moyenne)*(x2-moyenneDeg);
			}
		}
		Cov1=somme1/sommewi;

		sommewi=0;
		for(int i=0;i<TabDistanceDeg.size();i++)
		{

			int IndMin=i-2*ecart;
			if(IndMin<0)
				IndMin=0;

			int IndMax=i+2*ecart;
			if(IndMax>TabPoint.size()-1)
				IndMax=TabPoint.size()-1;

			x1=TabDistanceDeg[i];
			x2=TabDistance[IndMin];
			VBuff=TabPoint[IndMin]-TabPointDeg[i];
			maxBuff=sqrt(VBuff*VBuff);

			

			for(int j=IndMin+1;j<=IndMax;j++)
			{
				VBuff=TabPoint[j]-TabPointDeg[i];
				buff=sqrt(VBuff*VBuff);
				if(buff<maxBuff)
				{
					maxBuff=buff;
					x2=TabDistance[j];

				}

			}
			if(IsGauss==false)
			{
				somme2+=(x1-moyenneDeg)*(x2-moyenne);
				sommewi+=1;
			}
			else
			{
				Vector DistancePt=TabPointDeg[i]-pVertex->point();
				double distPt=sqrt(DistancePt*DistancePt);
				double wi=1/0.008/dim/sqrt(2*3.141592)*exp(-(distPt*distPt)/2/0.008/0.008/dim/dim);

				sommewi+=wi;
				somme2+=wi*(x1-moyenneDeg)*(x2-moyenne);
			}
		}
		Cov2=somme2/sommewi;

		if(Cov2>100000 || Cov2>100000)
			int klj=3;
		return (Cov2+Cov1)/2;


	}
	

	

	bool sphere_clip_vector(Point &O, double r,const Point &P, Vector &V)
    {

        Vector W = P - O ;
        double a = (V*V);
        double b = 2.0 * V * W ;
        double c = (W*W) - r*r ;
        double delta = b*b - 4*a*c ;

	

		if( a==0)
			return true ;

        if(delta < 0) {
            // Should not happen, but happens sometimes (numerical precision)
			
            return true ;
        }
        double t = (- b + std::sqrt(delta)) / (2.0 * a) ;
        if(t < 0.0) {
			
            // Should not happen, but happens sometimes (numerical precision)
            return true ;
        }
        if(t >= 1.0) {
            // Inside the sphere
            return false ;
        }

		if(t==0)
		{
			
			t=0.01;
		}

        V=V*t;

        return true ;
    }

	
	double Processroughness_per_vertex_curve(Polyhedron * PolyUsed,Vertex* pVertex,double radius,std::vector<double> &TabDistance,std::vector<Point> &TabPoint,double & moyenneRet,double dim,bool IsGauss = false)
	{
		std::set<Vertex*> vertices ;
        Point O = pVertex->point() ;
        std::stack<Vertex*> S ;
        S.push(pVertex) ;	
        vertices.insert(pVertex) ;
		
	
		int NbSommetInSphere=0;
		double SommeDistance=0;
	

        while(!S.empty())
		{
			Vertex* v = S.top() ;
            S.pop() ;
            Point P = v->point() ;
            Halfedge_around_vertex_circulator h = v->vertex_begin();
			Halfedge_around_vertex_circulator pHalfedgeStart = h;
			CGAL_For_all(h,pHalfedgeStart)
			{
                Point p1 = h->vertex()->point();
				Point p2 = h->opposite()->vertex()->point();
				Vector V = (p2-p1);
                if(v==pVertex || V * (P - O) > 0.0) 
				{
					double len_old = std::sqrt(V*V);
					bool isect = sphere_clip_vector(O, radius, P, V) ;
					double len_edge = std::sqrt(V*V);

					NbSommetInSphere++;
					///ici on prend en compte la distance map des sommets
					
					double DistancePondere;
					Point PPondere;
					if(len_old!=0)
					{
						DistancePondere=(1-len_edge/len_old)*h->vertex()->Kmax()+len_edge/len_old*h->opposite()->vertex()->Kmax();
						PPondere=p1+V;
					}
					else
					{
						DistancePondere=h->opposite()->vertex()->Kmax();
						PPondere=p2;
					}

					TabDistance.push_back(DistancePondere);
					TabPoint.push_back(PPondere);

					SommeDistance+=DistancePondere;

					if(!isect) 
					{
						
						Vertex_iterator w=h->opposite()->vertex();
                        if(vertices.find(&(*w)) == vertices.end())
						{
                            vertices.insert(&(*w)) ;
                            S.push(&(*w)) ;
                        }
                    }
					
				}
                
			}
			
		}

		double moyenne=0;
		double variance=0;

		if(IsGauss==false)
		{
			moyenne=SommeDistance/(double)NbSommetInSphere;

		
			

			variance=0;
			if(NbSommetInSphere!=0)
			{
				for(int i=0;i<NbSommetInSphere;i++)
					variance+=pow(TabDistance[i]-moyenne,2);

				variance=variance/(double)NbSommetInSphere;
				variance=sqrt(variance);
			}
		}
		else
		{//variance = 0.008
			SommeDistance=0;
			double SommeWi=0;
			for(int i=0;i<NbSommetInSphere;i++)
			{
				Vector DistancePt=TabPoint[i]-pVertex->point();
				double distPt=sqrt(DistancePt*DistancePt);
				double wi=1/0.008/dim/sqrt(2*3.141592)*exp(-(distPt*distPt)/2/0.008/0.008/dim/dim);
				SommeWi+=wi;
				SommeDistance+=TabDistance[i]*wi;
			}
			moyenne=SommeDistance/(double)SommeWi;

			
			for(int i=0;i<NbSommetInSphere;i++)
			{
				Vector DistancePt=TabPoint[i]-pVertex->point();
				double distPt=sqrt(DistancePt*DistancePt);
				double wi=1/0.008/dim/sqrt(2*3.141592)*exp(-(distPt*distPt)/2/0.008/0.008/dim/dim);

				variance+=wi*pow(TabDistance[i]-moyenne,2);
			}

				variance=variance/SommeWi;
				variance=sqrt(variance);

		}
			//variance=sqrt(variance);

			//variance=log(variance+10);
			



			//////////////////////
			pVertex->Kh(variance);
			pVertex->Kg(moyenne);
			
			pVertex->CourbureMoyenne=moyenne;
			pVertex->CourbureVariance=variance;

            if(variance<PolyUsed->MinNrmMeanCurvature())
						PolyUsed->MinNrmMeanCurvature(variance);

			if(variance>PolyUsed->MaxNrmMeanCurvature())
				PolyUsed->MaxNrmMeanCurvature(variance);

			if(moyenne<PolyUsed->MinNrmSalCurvature())
						PolyUsed->MinNrmSalCurvature(moyenne);

			if(moyenne>PolyUsed->MaxNrmSalCurvature())
				PolyUsed->MaxNrmSalCurvature(moyenne);

			 if(moyenne<PolyUsed->MinNrmGaussCurvature())
						PolyUsed->MinNrmGaussCurvature(moyenne);

			if(moyenne>PolyUsed->MaxNrmGaussCurvature())
				PolyUsed->MaxNrmGaussCurvature(moyenne);

			

			
			moyenneRet=moyenne;



			return variance;

	}

	

	

	

	void ComputeDistanceEcartNormalRoughnessPonderate(double Param,double &L1,double &L2,double &L3,double &L4)
	{
		double SommeDist=0;
		double SommeDistCarre=0;
		double SommeDist3=0;
		double SommeDist4=0;
		double Max=0;
		

		int NbVertex=0;

		double PolyDegradRoughMin=m_PolyDegrad.MinNrmMeanCurvature();
		double PolyDegradRoughMax=m_PolyDegrad.MaxNrmMeanCurvature();

		double PolyOriginalRoughMin=m_PolyOriginal->MinNrmMeanCurvature();
		double PolyOriginalRoughMax=m_PolyOriginal->MaxNrmMeanCurvature();


		m_PolyOriginal->MinNrmGaussCurvature(10000);
		m_PolyOriginal->MaxNrmGaussCurvature(0);

		int CMPT=0;
		Vertex_iterator	pVertexDeg = m_PolyDegrad.vertices_begin();
		for(Vertex_iterator	pVertex	= m_PolyOriginal->vertices_begin();
					pVertex	!= m_PolyOriginal->vertices_end();
					pVertex++)
		{
				

			/////////////////////:avec covariance///////////////////////////////////::
			double MoyX=pVertex->CourbureMoyenne;
			double MoyY=pVertexDeg->CourbureMoyenne;
			double SigX=pVertex->CourbureVariance;
			double SigY=pVertexDeg->CourbureVariance;
			double SigXY=pVertex->CourbureCoVariance;

			double angleRad;
			
			////minkowsky
			double fact1=(fabs(MoyX-MoyY))/(std::max(MoyX,MoyY)+1);//(2*MoyX*MoyY)/(MoyX*MoyX+MoyY*MoyY);
			double fact2=(fabs(SigX-SigY))/(std::max(SigX,SigY)+1);//2*SigX*SigY/(SigX*SigX+SigY*SigY);
			double fact3=(fabs(SigX*SigY-SigXY))/(SigX*SigY+1);//fabs(SigXY/(SigX*SigY));

		
			angleRad=pow((pow(fact1,3)+pow(fact2,3)+Param*pow(fact3,3))/(2.+Param),1./3.);
			
			
			if(MoyX<1)
				int h=2;

			CMPT++;



			

				SommeDist+=angleRad;
				SommeDistCarre+=angleRad*angleRad;
				SommeDist4+=angleRad*angleRad*angleRad*angleRad;
				SommeDist3+=angleRad*angleRad*angleRad;
				if(Max<angleRad)
					Max=angleRad;


				double Indice=angleRad;

				pVertex->Kg(Indice);
				if(Indice<m_PolyOriginal->MinNrmGaussCurvature())
					m_PolyOriginal->MinNrmGaussCurvature(Indice);

				if(Indice>m_PolyOriginal->MaxNrmGaussCurvature())
					m_PolyOriginal->MaxNrmGaussCurvature(Indice);

				NbVertex++;
				pVertexDeg++;
					

					

		}

		L2=sqrt(SommeDistCarre/(double)NbVertex);
		L1=SommeDist/(double)NbVertex;
		L4=pow(SommeDist4/(double)NbVertex,0.25);
		L3=pow(SommeDist3/(double)NbVertex,0.33333);


	}


	
};

#endif
