/*------------------------------------------------------------------------
Copyright LIRIS, M2DisCo 2009, Licence QPL
Author: Guillaume Lavoue
------------------------------------------------------------------------*/



#include "stdafx.h"

// CGAL
#include <CGAL/basic.h>
#include <CGAL/Simple_cartesian.h>
#include "enriched_polyhedron.h"
// CGAL IO
#include <CGAL/IO/Polyhedron_iostream.h>

//typedef
typedef float number_type;
typedef CGAL::Simple_cartesian<number_type> My_kernel;


//LIRIS

#include "io_obj.h"


//calcul distance
#include"Distance3D.h"
#include"normal_cycle.h"

#include <fstream>
#include <iostream>
#include <Afx.h>
using std::ifstream;


int _tmain(int argc, _TCHAR* argv[]) 
{
	if(argc!=4)
	{
		printf("\n\nUtilization:\n	MSDM.exe Name1 Name2 CurvRadius\n\n\nName1/2 <Names of the files to compare (.OFF or .OBJ)>\n\nCurvRadius <Radius for curvature calculation (percentage of the bounding box length)> 0.005 (higher sensitivity) or 0.01 (lower sensitivity) are recommended."); 
		printf("\n\nFor instance: MSDM.exe vase-lion.off vase-lion-noise.obj 0.01");
		printf("\n\nThe MSDM value is then displayed. This distance is not symetric.\n");
		printf("Warning: The two meshes have to own the same connectivity and the same vertex order in the files!\n");
	}
	else
	{
	
	char Nom1[1024];
	char Nom2[1024];
	char radiusC[256];
	strcpy(Nom1,argv[1]);
	strcpy(Nom2,argv[2]);
	strcpy(radiusC,argv[3]);
	float radius=atof(radiusC);

	////on verifie que le nom est valide
	FILE* test;
	test = fopen(Nom1,"r"); 
	if (test==NULL)
	{
		printf("The file %s does not exist\n\n",Nom1);
		return 0;
	}
	else
		fclose(test);

	test;
	test = fopen(Nom2,"r"); 
	if (test==NULL)
	{
		printf("The file %s does not exist\n\n",Nom2);
		return 0;
	}
	else
		fclose(test);

	printf("Calculation...");

	            
		//Les maillages d'entre
		Enriched_polyhedron<My_kernel,Enriched_items> *m_pMesh;
	
	
		//Les objets qui ralisent les traitement
		Distance3D<Enriched_polyhedron<My_kernel,Enriched_items> >*m_pDistance;
		
		//   Fin dclaration des variables de fonctionnement

		//construction du polyhedron d'entre
		m_pMesh = new Enriched_polyhedron<My_kernel,Enriched_items>;
		

		// Extension
		CString extension = Nom1;
		extension = extension.Right(3);
		extension.MakeLower();
		
		if(extension == "obj")
		{
			Parser_OBJ<My_kernel,Enriched_items> parser;///on enleve car I/O OFF
			parser.read(Nom1,m_pMesh);///on enleve car I/O OFF
		}
		else	
		{
			//lecture fichier OFF
			ifstream stream(Nom1);
			stream >> *m_pMesh;
		}

		//on normalize l'objet
		
		m_pMesh->compute_normals();
		////////////////////////////////////////////////////////////

		//allocate a new distance object
		m_pDistance = new Distance3D<Enriched_polyhedron<My_kernel,Enriched_items> >(m_pMesh);

		extension = Nom2;
		extension = extension.Right(3);
		extension.MakeLower();
		
		if(extension == "obj")
		{
			Parser_OBJ<My_kernel,Enriched_items> parser;///on enleve car I/O OFF
			parser.read(Nom2,&(m_pDistance->PolyDegrad()));///on enleve car I/O OFF
		}
		else	
		{
			//lecture fichier OFF
			ifstream stream(Nom2);
			stream >> m_pDistance->PolyDegrad();
		}

		//on normalize l'objet
		
		m_pDistance->PolyDegrad().compute_normals();

		m_pDistance->IsLoaded(true);
		////////////////////////////////////////////////////////////

		double maxdim=m_pDistance->m_PolyOriginal->getMaxDim();
		double maxdim2=m_pDistance->m_PolyDegrad.getMaxDim();

 

	Normal_cycle<Enriched_polyhedron<My_kernel,Enriched_items> > estimateur;
		estimateur.principal_curvature(*(m_pDistance->m_PolyOriginal),true,radius*maxdim);
		
	
	m_pDistance->m_PolyOriginal->KmaxKmean(maxdim);
	
 
	
	
	estimateur.principal_curvature((m_pDistance->m_PolyDegrad),true,radius*maxdim);

	m_pDistance->m_PolyDegrad.KmaxKmean(maxdim);
	
	
	

	m_pDistance->Processroughness_curve_Dual(0.015*maxdim,maxdim);

	

	double L1,L2,L4,L3;
	m_pDistance->ComputeDistanceEcartNormalRoughnessPonderate(0.5,L1,L2,L3,L4);

	printf("MSDM = %f\n\n",L3);
		
		delete m_pDistance;
		delete m_pMesh;
		
	}
	

	return 0;
}

